<?php
declare(strict_types=1);

namespace PPAM\PpamBasement\Utility;

use Psr\Http\Message\ServerRequestInterface;
use TYPO3\CMS\Core\TypoScript\FrontendTypoScript;

/**
 *
 */
class TypoScriptUtility

{
    /**
     * @param ServerRequestInterface $request
     * @return array
     */
    public static function getSetup(ServerRequestInterface $request): array
    {
        $frontendTypoScript = $request->getAttribute('frontend.typoscript');
        if ($frontendTypoScript instanceof FrontendTypoScript) {
            return $frontendTypoScript->getSetupArray();
        }

        return [];
    }

    /**
     * @param ServerRequestInterface $request
     * @param string $prefix
     * @param bool $stripPrefix
     * @return array
     */
    public static function getConstantsByPrefix(ServerRequestInterface $request, string $prefix, bool $stripPrefix = true): array
    {
        $constants = array_filter(
            self::getConstants($request),
            function (string $name) use ($prefix) {
                return strpos($name, $prefix . '.') === 0;
            },
            ARRAY_FILTER_USE_KEY
        );

        if ($stripPrefix === false) {
            return $constants;
        }

        $processedConstants = [];
        foreach ($constants as $name => $value) {
            $processedConstants[substr($name, strlen($prefix . '.'))] = $value;
        }

        return $processedConstants;
    }

    /**
     * @param ServerRequestInterface $request
     * @return array
     */
    public static function getConstants(ServerRequestInterface $request): array
    {
        $frontendTypoScript = $request->getAttribute('frontend.typoscript');
        if ($frontendTypoScript instanceof FrontendTypoScript) {
            return $frontendTypoScript->getFlatSettings();
        }
        return [];
    }

    /**
     * @param array $input
     * @return array
     */
    public static function unflatten(array $input): array
    {
        $output = [];
        foreach ($input as $key => $value) {
            $parts = explode('.', $key);
            $nested = &$output;
            while (count($parts) > 1) {
                $nested = &$nested[array_shift($parts)];
                if (!is_array($nested)) {
                    $nested = [];
                }
            }
            $nested[array_shift($parts)] = $value;
        }
        return $output;
    }
}